/*
 *  Copyright (C) 2006 Takeharu KATO
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <errno.h>
#include <glib.h>
#include "common.h"

#define IPMSG_INTERNAL_CODE "UTF-8"
#define IPMSG_PROTO_CODE    "CP932"
GStaticMutex convert_mutex = G_STATIC_MUTEX_INIT;
int
convert_string_internal(const char *string,const gchar **to_string){
  int rc=0;
  gchar *converted_string=NULL;
  gsize read_len;
  gsize write_len;
  GError *error_info=NULL;

  if ( (!string) || (!to_string) )
    return -EINVAL;

  g_static_mutex_lock(&convert_mutex);
  converted_string=g_convert((const gchar *)string,
			     -1, /* ヌルターミネート */
			     IPMSG_INTERNAL_CODE,
			     IPMSG_PROTO_CODE,
			     &read_len,
			     &write_len,
			     &error_info);
  rc=-EINVAL;
  if (!converted_string) {
    if (error_info) {
      err_out("%s\n",error_info->message);
      rc=error_info->code;
      g_error_free(error_info);
    }
    if (rc>0)
      rc=-rc;
    g_static_mutex_unlock(&convert_mutex);
    return rc;
  }

  *to_string=converted_string;
  g_static_mutex_unlock(&convert_mutex);
  return 0;
}

int
convert_string_ipmsg_proto(const char *string,const gchar **to_string){
  int rc=0;
  gchar *converted_string;
  gsize read_len;
  gsize write_len;
  GError *error_info=NULL;

  if ( (!string) || (!to_string) )
    return -EINVAL;

  g_static_mutex_lock(&convert_mutex);
  converted_string=g_convert((const gchar *)string,
			     -1, /* ヌルターミネート */
			     IPMSG_PROTO_CODE,
			     IPMSG_INTERNAL_CODE,
			     &read_len,
			     &write_len,
			     &error_info);
  rc=-EINVAL;
  if (!converted_string) {
    if (error_info) {
      err_out("%s\n",error_info->message);
      rc=error_info->code;
      g_error_free(error_info);
    }
    if (rc>0)
      rc=-rc;
    g_static_mutex_unlock(&convert_mutex);
    return rc;
  }

  *to_string=converted_string;
  g_static_mutex_unlock(&convert_mutex);

  return 0;
}

int
convert_string_ipmsg_locale(const char *string,const gchar **to_string){
  int rc=0;
  gchar *converted_string;
  gsize read_len;
  gsize write_len;
  GError *error_info=NULL;

  if ( (!string) || (!to_string) )
    return -EINVAL;

  g_static_mutex_lock(&convert_mutex);
  converted_string=g_locale_from_utf8((const gchar *)string,
			     -1, /* ヌルターミネート */
			     &read_len,
			     &write_len,
			     &error_info);
  if (!converted_string) {
    if (error_info) {
      err_out("%s\n",error_info->message);
      rc=error_info->code;
      g_error_free(error_info);
    }
    if (rc>0)
      rc=-rc;
    g_static_mutex_unlock(&convert_mutex);
    return rc;
  }
  *to_string=converted_string;
  g_static_mutex_unlock(&convert_mutex);
  return 0;
}

int
convert_string_ipmsg_filename(const char *string,const gchar **to_string){
  int rc=0;
  gchar *converted_string;
  gsize read_len;
  gsize write_len;
  GError *error_info=NULL;

  if ( (!string) || (!to_string) )
    return -EINVAL;

  g_static_mutex_lock(&convert_mutex);
  converted_string=g_filename_from_utf8((const gchar *)string,
			     -1, /* ヌルターミネート */
			     &read_len,
			     &write_len,
			     &error_info);

  rc=-EINVAL;
  if (!converted_string) {
    if (error_info) {
      err_out("%s\n",error_info->message);
      rc=error_info->code;
      g_error_free(error_info);
    }
    if (rc>0)
      rc=-rc;
    g_static_mutex_unlock(&convert_mutex);
    return rc;
  }

  *to_string=converted_string;
  g_static_mutex_unlock(&convert_mutex);
  return 0;
}
