/*
 *  Copyright (C) 2006 Takeharu KATO
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(G2IPMSG_PUBCRYPT_H)
#define G2IPMSG_PUBCRYPT_H
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <openssl/bn.h>
#include <openssl/rsa.h>
#include <openssl/pem.h>
#define RSA_KEYGEN_RETRY    10
#define RSA_KEY_INDEX_2048   0
#define RSA_KEY_INDEX_1024   1
#define RSA_KEY_INDEX_512    2
#define RSA_KEY_MAX          3
#define RSA_CAPS            (IPMSG_RSA_512|IPMSG_RSA_1024|IPMSG_RSA_2048)
//#define SIGN_CAPS           (IPMSG_SIGN_MD5)
#define SIGN_CAPS           (IPMSG_SIGN_MD5|IPMSG_SIGN_SHA1)

#define MD5_DIGEST_LEN       16
#define SHA1_DIGEST_LEN      20
#define MD5SHA1_DIGEST_LEN   36

#define RSA_DIR_INVALID_FLAGS (S_IWOTH|S_IWGRP) 
#define RSA_PRIVKEY_FNAME_PREFIX "priv-key-"
#define RSA_PUBKEY_FNAME_PREFIX  "pub-key-"

int pcrypt_get_rsa_key_length(unsigned long key_type,size_t *len);
int pcrypt_crypt_parse_anspubkey(const char *string,unsigned long *peer_cap,unsigned char **crypt_e,unsigned char **crypt_n);
int pcrypt_crypt_generate_anspubkey_string(unsigned long cap,const char **message_p);
int pcrypt_crypt_generate_getpubkey_string(unsigned long cap,const char **message_p);
int pcrypt_crypt_init_keys(void);
int pcrypt_crypt_release_keys(void);
int pcrypt_crypt_refer_rsa_key_with_index(int index,RSA **rsa_p);
int pcrypt_crypt_refer_rsa_key(unsigned long cap,RSA **rsa_p);
int pcrypt_encrypt_message(const unsigned long peer_cap,const char *peer_e,const char *peer_n,const char *plain,size_t plain_length,char **ret_buff,unsigned long *key_type);
int pcrypt_decrypt_message(unsigned long cap,const char *encrypted_skey,char **ret_buff,size_t *decrypted_len);
int pcrypt_sign(unsigned long cap,unsigned long sign_type,unsigned char *message, unsigned char **ret_buff);
int pcrypt_verify_sign(unsigned long cap,unsigned long sign_type,unsigned char *msg_str, 
		       unsigned char *sign,unsigned char *peer_e,unsigned char *peer_n);
int pcrypt_store_rsa_key(unsigned long key_name,pem_password_cb *cb);
int pcrypt_load_rsa_key(unsigned long key_name,pem_password_cb *cb);
int select_asymmetric_key(unsigned long peer_cap,RSA *rsa,unsigned long *selected_key);
int select_signature(unsigned long peer_cap,unsigned long *selected_algo,int speed);
#endif  /*  G2IPMSG_PUBCRYPT_H  */
