/*
 *  Copyright (C) 2006 Takeharu KATO
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <glib.h>
#include <openssl/rand.h>
#include "common.h"
static int
check_random_seed(void){
  int rc;
  unsigned short seed_in;
  int retry;

#if defined(HAVE_RAND_POLL)
  RAND_poll();
#endif  /*  HAVE_RAND_POLL  */

  for (rc=RAND_status(),retry=CRYPT_RND_MAX_RETRY;(!rc)&&(retry);rc=RAND_status(),--retry) {
    seed_in = rand() & 0xffff;      
    RAND_seed(&seed_in, sizeof(seed_in));
  }

  rc = (rc)?(0):(-EBUSY);

  return rc;
}
int 
add_timing_entropy(void) {
  /*  FIXME add implementation */
}
int
generate_rand(unsigned char *buf,size_t len){
  int rc=-EAGAIN;
  int retry=CRYPT_RND_MAX_RETRY;

  if (check_random_seed()) {
    err_out("Can not seed random\n");
    g_assert_not_reached();
  }

  do{
    memset(buf,0,len);    
    if (RAND_bytes(buf,len)) {
      rc=0;
      break;
    }
  }while( (rc) && (retry) );

  return rc;
}
