/*
 *  Copyright (C) 2006 Takeharu KATO
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(USERDB_H)
#define USERDB_H
#include <gnome.h>
#include <glib.h>
#include <string.h>
#include "udp.h"
#include "message.h"

#define make_entry_canonical(the_string) ((strncmp((the_string),HOSTLIST_DUMMY,1))?(the_string):(_("UnKnown")))


typedef struct _userdb{
  gchar *user;
  gchar *host;
  gchar *group;
  gchar *nickname;
  gchar *ipaddr;
  int    prio;
  unsigned long cap; /*  端末ケイパビリティ(エントリパケットのフラグ)  */
  unsigned long crypt_cap; /*  端末ケイパビリティ(エントリパケットのフラグ)  */
  gchar *pub_key_e;  /* hexフォーマット(bigendian)の文字列  */
  gchar *pub_key_n;  /* hexフォーマット(bigendian)の文字列  */
}userdb_t;
int userdb_init_userdb(void);
int userdb_del_user(const udp_con_t *con,const msg_data_t *msg);
int userdb_add_user(const udp_con_t *con,const msg_data_t *msg);
int userdb_update_user(const udp_con_t *con,const msg_data_t *msg);
int userdb_search_user(const char *user,const char *ipaddr,const userdb_t **entry_ref);
int userdb_search_user_by_addr(const char *ipaddr,const userdb_t **entry_ref);
int destroy_user_info(userdb_t *entry);
void userdb_update_group_list(GtkComboBox *widget);
int userdb_get_hostlist_string(int start, int *length, const char **ret_string,unsigned long *flags) ;
void userdb_print_user_list(void);
int update_users_on_message_window(GtkWidget *window,gboolean is_forced);
int userdb_invalidate_userdb(void);
int userdb_send_broad_cast(const udp_con_t *con,const char *msg,size_t len);
int userdb_replace_prio_by_addr(const char *ipaddr,int prio,gboolean need_notify);
void update_all_user_list_view(void);
int userdb_replace_public_key_by_addr(const char *ipaddr,const unsigned long peer_cap,const char *key_e,const char *key_n);
int userdb_get_public_key_by_addr(const char *ipaddr,unsigned long *cap_p,char **key_e,char **key_n);
int userdb_wait_public_key(const char *peer_addr,unsigned long *cap_p,char **key_e,char **key_n);
int userdb_get_crypt_cap_by_addr(const char *ipaddr,unsigned long *cap_p);
int userdb_get_basic_cap_by_addr(const char *ipaddr,unsigned long *cap_p);
int userdb_cleanup_userdb(void);
GList *refer_user_list(void);
GList *get_group_list(void);
#endif /* USERDB_H  */
